# authereum

> The [Authereum](https://authereum.com) library

[![Documentation](https://img.shields.io/badge/documentation-available-green.svg?style=flat)](https://docs.authereum.com/)
[![License](http://img.shields.io/badge/license-MIT-blue.svg)](https://raw.githubusercontent.com/@authereum/authereum/master/LICENSE)
[![dependencies Status](https://david-dm.org/@authereum/authereum/status.svg)](https://david-dm.org/@authereum/authereum)
[![NPM version](https://badge.fury.io/js/authereum.svg)](http://badge.fury.io/js/authereum)

## Install

```bash
npm install authereum
```

## Getting started

Initializing SDK:

```js
import Authereum from 'authereum'

const authereum = new Authereum(/* options */)

authereum.login()
```

Getting web3 provider:

```js
import Web3 from 'web3'
import Authereum from 'authereum'

const authereum = new Authereum(/* options */)
const provider = authereum.getProvider()
const web3 = new Web3(provider)
```

Using Ethers.js Signer:

```js
import { AuthereumSigner } from 'authereum'

const signer = new AuthereumSigner(/* options */)

const tx = {
  gasPrice: '0x2540be400',
  gasLimit: '0x186a0',
  to: '0x90f8bf6a479f320ead074411a4b0e7944ea8c9c1',
  value: '0x00',
  data: '0x',
}

const result = await signer.sendTransaction(tx)
console.log(result.hash)
```

Options:

- `mainnet`, `kovan`, `rinkeby`, `ropsten`, `goerli`


### CDN


[https://cdn.jsdelivr.net/npm/authereum@latest/authereum.js](jsDelivr CDN)


```html
<script src="https://cdn.jsdelivr.net/npm/authereum@latest/authereum.js"></script>
```

[https://unpkg.com/authereum@latest/authereum.js](unpkg CDN)

```html
<script src="https://unpkg.com/authereum@latest/authereum.js"></script>
```


## Full Documentation

Full documentation is available on our [docs page](https://docs.authereum.com/).


## Development

Watch and build:

```bash
npm run dev
```

Build:

```bash
npm run build
```

Lint:

```bash
npm run lint
```

## Test

```bash
npm test
```

## License

[MIT](LICENSE)
