import EventEmitter from 'eventemitter3';
import AuthereumProvider from './AuthereumProvider';
import StarkWareProvider from './StarkWareProvider';
import ZkSyncProvider from './ZkSyncProvider';
export interface ISignMessageBaseOptions {
    includeAuthSignature: true;
}
export interface ISignMessageOptions extends ISignMessageBaseOptions {
    personal: boolean;
}
export interface ISignTypedMessageOptions extends ISignMessageBaseOptions {
}
export default class Authereum extends EventEmitter {
    private _debug;
    private _apiKey;
    private _apiUri;
    private _rpcUri;
    private _webUri;
    private _xsUri;
    private _notifierUri;
    private _networkName;
    private _networkId;
    private _redirectUri;
    private _provider;
    private _iframeConnection;
    private _childIframeConnection;
    private _iframe;
    private _notificationIframe;
    private _notifier;
    private _disableNotifications;
    private _cacheReady;
    private _cachedLoginKeyAddress;
    private _cachedAccountAddress;
    private _cachedIsAuthenticated;
    private _iframeReady;
    private _configReady;
    private _iframeConnecting;
    private _readyEmitted;
    private _widgetEnabled;
    private _iframeStyle;
    private _notificationIframeStyle;
    private _blockedPopupRedirect;
    private _forceRedirect;
    private _heartbeatStarted;
    private _backoffTimeout;
    private _userConfig;
    private _asymmetricEncryptionKey;
    private _authereumQueryParams;
    private _injectedElements;
    private _notificationsMap;
    private _destroyed;
    private _starkProvider;
    private _zkSyncProvider;
    private _isPreload;
    private _disableGoogleAnalytics;
    private _eventListeners;
    constructor(config?: any);
    private _checkCachedInstance;
    private _merge;
    private _init;
    private _initConfig;
    private _setConfig;
    private _setupCache;
    _handleWindowFocus: () => Promise<any>;
    private _resetCache;
    private _injectScript;
    private _setupResizeListener;
    private _setupFocusListener;
    private _setupActionKeyListener;
    private _teardownEventListeners;
    private _handleActionKeyPress;
    private _handleResize;
    private _handleResizeThrottled;
    private _handleResizeDebounced;
    private _cacheLoginKeyAddress;
    private _cacheAccountAddress;
    private _cacheIsAuthenticated;
    login: (...args: any[]) => Promise<unknown>;
    authenticate: () => Promise<unknown>;
    isAuthenticated: () => Promise<any>;
    logout: () => Promise<any>;
    getAuthenticatedAccount: () => Promise<any>;
    getDappKey: () => Promise<any>;
    getLoginKey: () => Promise<any>;
    signMessage: (...args: any[]) => Promise<unknown>;
    signPersonalMessage: (...args: any[]) => Promise<unknown>;
    signWithDappKey: (...args: any[]) => Promise<unknown>;
    signWithLoginKey: (...args: any[]) => Promise<unknown>;
    signTypedMessage: (...args: any[]) => Promise<unknown>;
    signTypedMessageWithDappKey: (...args: any[]) => Promise<unknown>;
    signMessageWithAdminKey: (...args: any[]) => Promise<unknown>;
    signMessageWithSigningKey: (...args: any[]) => Promise<unknown>;
    getSigningKeyAddress: (message: any) => Promise<any>;
    getLoginKeyAuthSignature: () => Promise<any>;
    getLoginKeyRestrictionsData: () => Promise<any>;
    getSigningKeyAuthSignature: () => Promise<any>;
    getAccountAddress: () => Promise<any>;
    getBalance: () => Promise<any>;
    getRpcUri: () => Promise<string>;
    getNetworkId: () => Promise<any>;
    getNetworkName: () => Promise<any>;
    getCachedNetworkName: () => string;
    getProvider: () => AuthereumProvider;
    getStarkProvider: () => StarkWareProvider;
    getZkSyncProvider: () => ZkSyncProvider;
    private _handleError;
    signTransaction: (...args: any[]) => Promise<unknown>;
    signTransactionWithLoginKey: (...args: any[]) => Promise<unknown>;
    sendTransaction: (...args: any[]) => Promise<unknown>;
    sendTransactionBatch: (...args: any[]) => Promise<unknown>;
    estimateGas: (userTransaction: any) => Promise<any>;
    estimateGasBatch: (userTransactions: any[]) => Promise<any>;
    isReady: () => boolean;
    updateConfig: (config: any) => void;
    getConfig: () => {
        apiUri: string;
        rpcUri: string;
        webUri: string;
        xsUri: string;
        networkName: string;
        networkId: string;
        disableNotifications: boolean;
    };
    showWidget: (enabled?: boolean) => Promise<boolean>;
    hideWidget: () => Promise<boolean>;
    widgetEnabled: () => boolean;
    addFunds: (opts: any) => Promise<any>;
    private _openUrl;
    private _tilIframeReady;
    _tilConfigReady: () => Promise<any>;
    _tilReady: () => Promise<any>;
    private _emitReadyEvent;
    private _connectToChildIframe;
    private _startHeartbeat;
    private _heartbeat;
    private _ping;
    private _getOrCreateMainIframe;
    private _constructIframe;
    private _getConfigFromChild;
    private _postVersionToChild;
    private _postConfigToChild;
    private _loadAsymmetricEncryptionKeyFromChild;
    _postMessageToChild: (payload: any) => Promise<any>;
    private _bodyContainsIframe;
    private _handleChildLogoutEvent;
    private _handleChildDappKeyExpired;
    private _handlePostMessage;
    private _handlePostMessageLog;
    private _getWindowSize;
    private _getWindowWidth;
    private _getWindowHeight;
    _setIframePositionLeft: (x: number) => void;
    _setIframeWidth: (width: number) => void;
    _setIframeHeight: (height: number) => void;
    private _refreshWidget;
    private _refreshNotificationWidget;
    private _loginKeyExists;
    private _accountAddressExists;
    private _getAccountAdminKeyNonce;
    private _getAccountAuthKeyNonce;
    private _executeLoginKeyMetaTx;
    private _loadLoginKey;
    private _getAuthenticateUri;
    private _getSignWithAdminKeyUri;
    private _openPopupOrRedirect;
    addEventListener: (type: string, listener: any, useCapture?: boolean) => void;
    private _getAuthereumPopupWindow;
    private _isAuthereumPopupBlocked;
    _showConfirmPopupOrRedirectModal: () => Promise<any>;
    private _signLoginKeyRequest;
    private _tilDomReady;
    version: () => string;
    static version: () => string;
    isAuthenticatedSync: () => boolean;
    getTransactionReceipt: (txHash: string) => Promise<any>;
    waitForTransactionReceipt: (txHash: string) => Promise<any>;
    hasRecoveryEnabled: () => Promise<any>;
    isContractDeployed: (accountAddress: string) => Promise<any>;
    private _loadSigningKey;
    private _logAnalyticsEvent;
    private _setupSigningKey;
    private _cacheAuthereumQueryParams;
    destroy: () => Promise<boolean>;
    isDestroyed: () => boolean;
    getAddressQrCodeDataUri: (address?: string) => Promise<any>;
    _loadNotificationsIframe: (url: string) => Promise<void>;
    isPreload(): boolean;
    _setDebug: (enabled: boolean) => Promise<void>;
}
